# -*- encoding: utf-8 -*-
require 'spec_helper'
#コマ絵
describe PanelPicture do
  before do
    @admin = FactoryGirl.create :admin
    @user = FactoryGirl.create( :user_yas)
    @author = FactoryGirl.create :author, :user_id => @user.id
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    @other_user = FactoryGirl.create( :user_yas)
    @other_author = FactoryGirl.create :author, :user_id => @other_user.id
    @other_artist = FactoryGirl.create :artist_yas, :author_id => @other_author.id
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
    @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
    @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
    @panel = FactoryGirl.create :panel, :author_id => @author.id
  end
  
  describe '検証に於いて' do
    before do
      @pp = FactoryGirl.build :panel_picture, :panel_id => @panel.id, :picture_id => @p.id,
        :width => @p.width, :height => @p.height
      Picture.any_instance.stub(:flag_reverse).with(any_args).and_return(0)
      Picture.any_instance.stub(:flag_resize).with(any_args).and_return(0)
      Picture.any_instance.stub(:flag_sync_vh).with(any_args).and_return(0)
    end
    
    context 'オーソドックスなデータのとき' do
      it '下限データが通る' do
        @pp.link = 'http://test.jp/'
        @pp.x = -99999
        @pp.y = -99999
        @pp.z = 1
        @pp.t = 0
        @pp.width = -99999
        @pp.height = -99999
        @pp.should be_valid
      end
      it '上限データが通る' do
        @pp.link = 'http://test.jp/aaaaa' + 'a' * 180
        @pp.x = 99999
        @pp.y = 99999
        @pp.z = 99999
        @pp.t = 99999
        @pp.width = 99999
        @pp.height = 99999
        @pp.should be_valid
      end
    end
    
    context 'panel_idを検証するとき' do
      #ネストの保存はnilを許可しなければならないので数値チェックだけ
      it '数値でなければ失敗する' do
        @pp.panel_id = 'a'
        @pp.should_not be_valid
      end
    end
    context 'linkを検証するとき' do
      it 'nullでも通る' do
        @pp.link = ''
        @pp.should be_valid
      end
      it '201文字以上なら失敗する' do
        @pp.link = 'a'*201
        @pp.should_not be_valid
      end
      it 'url形式でないなら失敗する' do
        @pp.link = 'a'*200
        @pp.should_not be_valid
      end
    end
    context 'xを検証するとき' do
      it 'nullなら失敗する' do
        @pp.x = nil
        @pp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @pp.x = 'a'
        @pp.should_not be_valid
      end
      it '0なら通る' do
        @pp.x = '0'
        @pp.should be_valid
      end
      it '負でも通る' do
        @pp.x = -1
        @pp.should be_valid
      end
    end
    context 'yを検証するとき' do
      it 'nullなら失敗する' do
        @pp.y = nil
        @pp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @pp.y = 'a'
        @pp.should_not be_valid
      end
      it '0なら通る' do
        @pp.y = '0'
        @pp.should be_valid
      end
      it '負でも通る' do
        @pp.y = -1
        @pp.should be_valid
      end
    end
    context 'widthを検証するとき' do
      it 'nullなら失敗する' do
        @pp.width = nil
        @pp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @pp.width = 'a'
        @pp.should_not be_valid
      end
      it '0なら失敗する' do
        @pp.width = '0'
        @pp.should_not be_valid
      end
      context  '反転許可のとき' do
        before do
        end
        it '負でも通る' do
          @pp.width = -1
          @pp.should be_valid
        end
      end
      context  '反転禁止のとき' do
        before do
          Picture.any_instance.stub(:flag_reverse).with(any_args).and_return(-1)
        end
        it '負なら失敗する' do
          @pp.width = -1
          @pp.should_not be_valid
        end
      end
      context  'サイズ変更許可のとき' do
        before do
        end
        it '実素材のサイズと違っても通る' do
          @pp.width = @p.width-1
          @pp.should be_valid
        end
      end
      context  'サイズ変更禁止のとき' do
        before do
          Picture.any_instance.stub(:flag_resize).with(any_args).and_return(-1)
        end
        it '実素材のサイズと違うなら失敗する' do
          @pp.width = @p.width-1
          @pp.should_not be_valid
        end
      end
      context  '縦横比変更許可のとき' do
        before do
        end
        it '実素材の縦横比と違っても通る' do
          @pp.width = @p.width / 2
          @pp.should be_valid
        end
      end
      context  '縦横比変更禁止のとき' do
        before do
          Picture.any_instance.stub(:flag_sync_vh).with(any_args).and_return(-1)
        end
        it '実素材の縦横比と違うなら失敗する' do
          @pp.width = @p.width / 2
          @pp.should_not be_valid
        end
      end
    end
    context 'heightを検証するとき' do
      it 'nullなら失敗する' do
        @pp.height = nil
        @pp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @pp.height = 'a'
        @pp.should_not be_valid
      end
      it '0なら失敗する' do
        @pp.height = '0'
        @pp.should_not be_valid
      end
      it '負でも通る' do
        @pp.height = -1
        @pp.should be_valid
      end
      context  '反転許可のとき' do
        before do
        end
        it '負でも通る' do
          @pp.height = -1
          @pp.should be_valid
        end
      end
      context  '反転禁止のとき' do
        before do
          Picture.any_instance.stub(:flag_reverse).with(any_args).and_return(-1)
        end
        it '負なら失敗する' do
          @pp.height = -1
          @pp.should_not be_valid
        end
      end
      context  'サイズ変更許可のとき' do
        before do
        end
        it '実素材のサイズと違っても通る' do
          @pp.height = @p.height-1
          @pp.should be_valid
        end
      end
      context  'サイズ変更禁止のとき' do
        before do
          Picture.any_instance.stub(:flag_resize).with(any_args).and_return(-1)
        end
        it '実素材のサイズと違うなら失敗する' do
          @pp.height = @p.height-1
          @pp.should_not be_valid
        end
      end
      context  '縦横比変更許可のとき' do
        before do
        end
        it '実素材の縦横比と違っても通る' do
          @pp.height = @p.height / 2
          @pp.should be_valid
        end
      end
      context  '縦横比変更禁止のとき' do
        before do
          Picture.any_instance.stub(:flag_sync_vh).with(any_args).and_return(-1)
        end
        it '実素材の縦横比と違うなら失敗する' do
          @pp.height = @p.height / 2
          @pp.should_not be_valid
        end
      end
    end
    context 'zを検証するとき' do
      it 'nullなら失敗する' do
        @pp.z = nil
        @pp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @pp.z = 'a'
        @pp.should_not be_valid
      end
      it '負なら失敗する' do
        @pp.z = -1
        @pp.should_not be_valid
      end
      it '負なら失敗する' do
        @pp.z = 0
        @pp.should_not be_valid
      end
    end
    context 'tを検証するとき' do
      it 'nullなら失敗する' do
        @pp.t = nil
        @pp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @pp.t = 'a'
        @pp.should_not be_valid
      end
      it '負なら失敗する' do
        @pp.t = -1
        @pp.should_not be_valid
      end
    end
    context 'picture_idを検証するとき' do
      it 'nullなら失敗する' do
        @pp.picture_id = nil
        @pp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @pp.picture_id = 'a'
        @pp.should_not be_valid
      end
      it '存在する実素材でなければ失敗する' do
        @pp.picture_id = 0
        @pp.should_not be_valid
      end
    end
  end
  
  describe '文字コード検証に於いて' do
    before do
      @pp = FactoryGirl.build :panel_picture, :panel_id => @panel.id, :picture_id => @p.id,
        :width => @p.width, :height => @p.height
    end
    
    context 'linkを検証するとき' do
      it 'Shift JISなら失敗する' do
        @pp.link = "\x83G\x83r\x83]\x83D"
        lambda{
          @pp.valid_encode
        }.should raise_error(Pettanr::BadRequest)
      end
    end
    
    context 'captionを検証するとき' do
      it 'Shift JISなら失敗する' do
        @pp.caption = "\x83G\x83r\x83]\x83D"
        lambda{
          @pp.valid_encode
        }.should raise_error(Pettanr::BadRequest)
      end
    end
  end
  
  describe '閲覧許可に於いて' do
    before do
      @pp = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :picture_id => @p.id,
        :width => @p.width, :height => @p.height
    end
    context 'オープンモードのとき' do
      before do
        MagicNumber['run_mode'] = 0
      end
      it '自身にゲスト用ロールチェックを問い合わせしている' do
        PanelPicture.any_instance.stub(:guest_role_check).and_return(true)
        PanelPicture.any_instance.should_receive(:guest_role_check).with(any_args).exactly(1)
        r = @pp.visible?([@author])
      end
      it 'ゲスト用ロールチェックが失敗したとき、falseを返す' do
        PanelPicture.any_instance.stub(:guest_role_check).and_return(false)
        r = @pp.visible?([@author])
        r.should be_false
      end
    end
    context 'クローズドモードのとき' do
      before do
        MagicNumber['run_mode'] = 1
      end
      it '自身に読者用ロールチェックを問い合わせしている' do
        PanelPicture.any_instance.stub(:reader_role_check).and_return(true)
        PanelPicture.any_instance.should_receive(:reader_role_check).with(any_args).exactly(1)
        r = @pp.visible?([@author])
      end
      it '読者用ロールチェックが失敗したとき、falseを返す' do
        PanelPicture.any_instance.stub(:reader_role_check).and_return(false)
        r = @pp.visible?([@author])
        r.should be_false
      end
    end
    context '事前チェックする' do
      before do
        MagicNumber['run_mode'] = 1
        PanelPicture.any_instance.stub(:reader_role_check).and_return(true)
      end
      it '自身のコマに所持判定を問い合わせしている' do
        Panel.any_instance.stub(:own?).and_return(true)
        Panel.any_instance.should_receive(:own?).with(any_args).exactly(1)
        r = @pp.visible?([@author])
      end
      it '自身のコマに閲覧許可を問い合わせしている' do
        Panel.any_instance.stub(:own?).and_return(false)
        Panel.any_instance.stub(:visible?).and_return(true)
        Panel.any_instance.should_receive(:visible?).with(any_args).exactly(1)
        r = @pp.visible?([@author])
      end
    end
    context 'つつがなく終わるとき' do
      before do
        MagicNumber['run_mode'] = 1
        Panel.any_instance.stub(:reader_role_check).and_return(true)
      end
      it '自分のコマのコマ絵なら許可する' do
        Panel.any_instance.stub(:own?).and_return(true)
        Panel.any_instance.stub(:visible?).and_return(false)
        r = @pp.visible?([@author])
        r.should be_true
      end
      it '他人の非公開コマのコマ絵なら許可しない' do
        Panel.any_instance.stub(:own?).and_return(false)
        Panel.any_instance.stub(:visible?).and_return(false)
        r = @pp.visible?([@author])
        r.should be_false
      end
      it '他人のコマのコマ絵でも公開なら許可する' do
        Panel.any_instance.stub(:own?).and_return(false)
        Panel.any_instance.stub(:visible?).and_return(true)
        r = @pp.visible?([@author])
        r.should be_true
      end
    end
  end
  
  
  describe '一覧取得に於いて' do
    before do
      @pp = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :picture_id => @p.id,
        :width => @p.width, :height => @p.height
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        PanelPicture.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        PanelPicture.page().should eq 1
      end
      it '0以下の場合は1になる' do
        PanelPicture.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        PanelPicture.page_size('7').should eq 7
      end
      it 'nilの場合はPanelPicture.default_page_sizeになる' do
        PanelPicture.page_size().should eq PanelPicture.default_page_size
      end
      it '0以下の場合はPanelPicture.default_page_sizeになる' do
        PanelPicture.page_size('0').should eq PanelPicture.default_page_size
      end
      it 'PanelPicture.max_page_sizeを超えた場合はPanelPicture.max_page_sizeになる' do
        PanelPicture.page_size('1000').should eq PanelPicture.max_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        PanelPicture.stub(:list_opt).with(any_args).and_return({:include => :panel})
        PanelPicture.should_receive(:list_opt).with(any_args).exactly(1)
        r = PanelPicture.list
      end
    end
    it 'リストを返す' do
      r = PanelPicture.list
      r.should eq [@pp]
    end
    it '時系列で並んでいる' do
      #公開されたコマのコマ絵は(他人のコマ絵であっても)含んでいる
      hc = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
      npl = FactoryGirl.create :panel_picture, :panel_id => hc.id, :t => 0, :picture_id => @p.id,
        :width => @p.width, :height => @p.height, :updated_at => Time.now + 100
      r = PanelPicture.list
      r.should eq [npl, @pp]
    end
    it '非公開のコマのコマ絵は自分のコマ絵であっても含まない' do
      hpl = FactoryGirl.create :panel, :author_id => @author.id, :publish => 0
      npl = FactoryGirl.create :panel_picture, :panel_id => hpl.id, :picture_id => @p.id,
        :width => @p.width, :height => @p.height
      pl = PanelPicture.list
      pl.should eq [@pp]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @npl2 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 1, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 100
        @npl3 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 2, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 200
        @npl4 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 3, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 300
        @npl5 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 4, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 400
        PanelPicture.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        pl = PanelPicture.list
        pl.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        pl = PanelPicture.list(1)
        pl.should eq [@npl5, @npl4]
      end
      it 'page=2なら中間2件を返す' do
        pl = PanelPicture.list(2)
        pl.should eq [@npl3, @npl2]
      end
      it 'page=3なら先頭1件を返す' do
        pl = PanelPicture.list(3)
        pl.should eq [@pp]
      end
    end
    context 'DBに5件あって1ページの件数を0件に変えたとして' do
      before do
        @npl2 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 1, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 100
        @npl3 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 2, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 200
        @npl4 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 3, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 300
        @npl5 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 4, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 400
        PanelPicture.stub(:default_page_size).and_return(2)
      end
      it '通常は全件(5件)を返す' do
        r = PanelPicture.list 5, 0
        r.should have(5).items 
      end
    end
  end
  describe '一覧取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = PanelPicture.list_opt
      r.has_key?(:include).should be_true
    end
    it '2つの項目を含んでいる' do
      r = PanelPicture.list_opt[:include]
      r.should have(2).items
    end
    it 'コマを含んでいる' do
      r = PanelPicture.list_opt[:include]
      r.has_key?(:panel).should be_true
    end
      it 'コマは作家を含んでいる' do
        r = PanelPicture.list_opt[:include]
        r[:panel].has_key?(:author).should be_true
      end
    it '実素材を含んでいる' do
      r = PanelPicture.list_opt[:include]
      r.has_key?(:picture).should be_true
    end
      it '実素材は絵師を含んでいる' do
        r = PanelPicture.list_opt[:include]
        r[:picture].has_key?(:artist).should be_true
      end
      it '実素材はライセンスを含んでいる' do
        r = PanelPicture.list_opt[:include]
        r[:picture].has_key?(:license).should be_true
      end
  end
  describe 'json一覧出力オプションに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
      @sbt = FactoryGirl.create :speech_balloon_template
      @panel = FactoryGirl.create :panel, :author_id => @author.id, :publish => 1
      @pp = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :picture_id => @p.id,
        :width => @p.width, :height => @p.height
    end
    it 'コマを含んでいる' do
      r = PanelPicture.list.to_json PanelPicture.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('panel').should be_true
    end
      it 'コマは作家を含んでいる' do
        r = PanelPicture.list.to_json PanelPicture.list_json_opt
        j = JSON.parse r
        i = j.first
        s = i['panel']
        s.has_key?('author').should be_true
      end
    it '実素材を含んでいる' do
      r = PanelPicture.list.to_json PanelPicture.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('picture').should be_true
    end
      it '実素材は絵師を含んでいる' do
        r = PanelPicture.list.to_json PanelPicture.list_json_opt
        j = JSON.parse r
        i = j.first
        s = i['picture']
        s.has_key?('artist').should be_true
      end
      it '実素材はライセンスを含んでいる' do
        r = PanelPicture.list.to_json PanelPicture.list_json_opt
        j = JSON.parse r
        i = j.first
        s = i['picture']
        s.has_key?('license').should be_true
      end
  end
  
  describe '自分のコマ絵一覧取得に於いて' do
    before do
      @pp = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :picture_id => @p.id,
        :width => @p.width, :height => @p.height
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        PanelPicture.stub(:list_opt).with(any_args).and_return({:include => :panel})
        PanelPicture.should_receive(:list_opt).with(any_args).exactly(1)
        r = PanelPicture.mylist @author
      end
    end
    it 'リストを返す' do
      pl = PanelPicture.mylist @author
      pl.should eq [@pp]
    end
    it '時系列で並んでいる' do
      npl = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 1, :picture_id => @p.id,
        :width => @p.width, :height => @p.height, :updated_at => Time.now + 100
      pl = PanelPicture.mylist @author
      pl.should eq [npl, @pp]
    end
    it '他人のコマのコマ絵は公開コマでも含まない' do
      hpl = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
      npl = FactoryGirl.create :panel_picture, :panel_id => hpl.id, :picture_id => @p.id,
        :width => @p.width, :height => @p.height
      pl = PanelPicture.mylist @author
      pl.should eq [@pp]
    end
    it '自分のコマのコマ絵は非公開コマでも含んでいる' do
      pl = FactoryGirl.create :panel, :author_id => @author.id, :publish => 0
      ni = FactoryGirl.create :panel_picture, :panel_id => pl.id, :picture_id => @p.id,
        :width => @p.width, :height => @p.height, :updated_at => Time.now + 100
      r = PanelPicture.mylist @author
      r.should eq [ni, @pp]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @npl2 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 1, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 100
        @npl3 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 2, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 200
        @npl4 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 3, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 300
        @npl5 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 4, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 400
      end
      it '通常は2件を返す' do
        r = PanelPicture.mylist @author, 1, 2
        r.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        r = PanelPicture.mylist(@author, 1, 2)
        r.should eq [@npl5, @npl4]
      end
      it 'page=2なら中間2件を返す' do
        r = PanelPicture.mylist(@author, 2, 2)
        r.should eq [@npl3, @npl2]
      end
      it 'page=3なら先頭1件を返す' do
        r = PanelPicture.mylist(@author, 3, 2)
        r.should eq [@pp]
      end
    end
    context 'DBに5件あって1ページの件数を0件に変えたとして' do
      before do
        @npl2 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 1, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 100
        @npl3 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 2, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 200
        @npl4 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 3, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 300
        @npl5 = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :t => 4, :picture_id => @p.id,
          :width => @p.width, :height => @p.height, :updated_at => Time.now + 400
        Author.stub(:default_panel_picture_page_size).and_return(2)
      end
      it '通常は全件(5件)を返す' do
        r = PanelPicture.mylist @author, 5, 0
        r.should have(5).items 
      end
    end
  end
  
  describe '単体取得に於いて' do
    before do
      @pp = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :picture_id => @p.id,
        :width => @p.width, :height => @p.height
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        PanelPicture.stub(:show_opt).with(any_args).and_return({:include => :panel})
        PanelPicture.should_receive(:show_opt).with(any_args).exactly(1)
        r = PanelPicture.show @pp.id, @author
      end
      it '閲覧許可を問い合わせている' do
        PanelPicture.any_instance.stub(:visible?).with(any_args).and_return(true)
        PanelPicture.any_instance.should_receive(:visible?).with(any_args).exactly(1)
        r = PanelPicture.show @pp.id, @author
      end
    end
    it '指定のコマ絵を返す' do
      PanelPicture.any_instance.stub(:visible?).and_return(true)
      pl = PanelPicture.show @pp.id, @author
      pl.should eq @pp
    end
    context '閲覧許可が出なかったとき' do
      it '403Forbidden例外を返す' do
        PanelPicture.any_instance.stub(:visible?).and_return(false)
        lambda{
          PanelPicture.show @pp.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しないコマ絵を開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          PanelPicture.show 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  describe '単体取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = PanelPicture.show_opt
      r.has_key?(:include).should be_true
    end
    it '2つの項目を含んでいる' do
      r = PanelPicture.show_opt[:include]
      r.should have(2).items
    end
    it 'コマを含んでいる' do
      r = PanelPicture.show_opt[:include]
      r.has_key?(:panel).should be_true
    end
      it 'コマは作家を含んでいる' do
        r = PanelPicture.show_opt[:include]
        r[:panel].has_key?(:author).should be_true
      end
    it '実素材を含んでいる' do
      r = PanelPicture.show_opt[:include]
      r.has_key?(:picture).should be_true
    end
      it '実素材は絵師を含んでいる' do
        r = PanelPicture.show_opt[:include]
        r[:picture].has_key?(:artist).should be_true
      end
      it '実素材はライセンスを含んでいる' do
        r = PanelPicture.show_opt[:include]
        r[:picture].has_key?(:license).should be_true
      end
  end
  describe 'json単体出力オプションに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
      @sbt = FactoryGirl.create :speech_balloon_template
      @panel = FactoryGirl.create :panel, :author_id => @author.id, :publish => 1
      @pp = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :picture_id => @p.id,
        :width => @p.width, :height => @p.height
    end
    it 'コマを含んでいる' do
      r = PanelPicture.show(@pp.id, @author).to_json PanelPicture.show_json_opt
      j = JSON.parse r
      i = j
      i.has_key?('panel').should be_true
    end
      it 'コマは作家を含んでいる' do
        r = PanelPicture.show(@pp.id, @author).to_json PanelPicture.show_json_opt
        j = JSON.parse r
        i = j
        s = i['panel']
        s.has_key?('author').should be_true
      end
    it '実素材を含んでいる' do
      r = PanelPicture.show(@pp.id, @author).to_json PanelPicture.show_json_opt
      j = JSON.parse r
      i = j
      i.has_key?('picture').should be_true
    end
      it '実素材は絵師を含んでいる' do
        r = PanelPicture.show(@pp.id, @author).to_json PanelPicture.show_json_opt
        j = JSON.parse r
        i = j
        s = i['picture']
        s.has_key?('artist').should be_true
      end
      it '実素材はライセンスを含んでいる' do
        r = PanelPicture.show(@pp.id, @author).to_json PanelPicture.show_json_opt
        j = JSON.parse r
        i = j
        s = i['picture']
        s.has_key?('license').should be_true
      end
  end
end
